package com.grupogodo.corp.projects.optimus.composers;

import javax.annotation.PostConstruct;

import lombok.*;

import org.springframework.beans.factory.annotation.Autowired;
import org.zkoss.spring.SpringUtil;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.select.annotation.Listen;
import org.zkoss.zk.ui.select.annotation.Wire;
import org.zkoss.zkplus.databind.DataBinder;
import org.zkoss.zul.*;

import com.grupogodo.corp.libs.jpa.core.BaseEntity;
import com.grupogodo.corp.libs.springzk.spring.*;
import com.grupogodo.corp.libs.zkcomponents.components.growl.Growl;
import com.grupogodo.corp.libs.zkcomponents.components.growl.Growl.Severity;
import com.grupogodo.corp.libs.zkcomponents.components.listbox.GListBox;
import com.grupogodo.corp.projects.optimus.composers.popups.ProcessPopupComposer;
import com.grupogodo.corp.projects.optimus.entities.*;
import com.grupogodo.corp.projects.optimus.entities.Group;
import com.grupogodo.corp.projects.optimus.services.UserPermissionService;

@org.springframework.stereotype.Component
public class GroupDetailComposer extends ZKModelAwareComposer<Component> {

    private static final long serialVersionUID = -8052122821039164748L;

    public static Include openForm(TreeNode<BaseEntity> treeNode, ZKCrossComposer<BaseEntity> processComposer) {
        GroupDetailComposer groupDetailComposer = SpringUtil.getApplicationContext().getBean(GroupDetailComposer.class);

        groupDetailComposer.setProcessComposer(processComposer);
        groupDetailComposer.setGroup((Group) treeNode.getData());
        groupDetailComposer.setTreeNode(treeNode);
        
        Include include = new Include("/WEB-INF/views/groupForm.zul");
        return include;
    }
    
    @Setter(AccessLevel.PRIVATE)
    private ZKCrossComposer<BaseEntity> processComposer;
    
    @Getter
    private ListModelList<UserPermission> permissionModel;
    
    @Getter
    private ListModelList<ProcessDefinition> processDefinitionModel;
    
    @Autowired
    private UserPermissionService permissionService;

    @Getter @Setter
    private Group group;
    
    @Setter
    private TreeNode<BaseEntity> treeNode;
    
    @Wire
    GListBox parametersListbox;
    
    @PostConstruct
    @Reload
    public void init() {
        permissionModel = new ListModelList<UserPermission>();
        processDefinitionModel = new ListModelList<ProcessDefinition>();
    }
    
    @Override
    public void doAfterCompose(Component component) throws Exception {
        super.doAfterCompose(component);
        permissionModel.clear();
        processDefinitionModel.clear();
        permissionModel.addAll(group.getPermissions());
        processDefinitionModel.addAll(group.getProcesses());
    }

    @Listen("onDeletePermission = #permissionGrid")
    public void onDeletePermission(UserPermission permission) {
        permissionModel.remove(permission);
    }

    @Listen("onClick = #addAuthorityButton")
    public void addRole() {
        permissionModel.add(new UserPermission("Permission name", "Description permission"));
    }

    // guardar los datos de groups
    @Listen("onClick = #saveButton")
    public void save(DataBinder binder) {
        
        group.getPermissions().clear();
        group.getPermissions().addAll(permissionModel);
        
        group.getProcesses().clear();
        group.getProcesses().addAll(processDefinitionModel);
        
        processComposer.onSubmit(group, binder);
        
        Growl.showMessageI18n("ui.message.saveOk", "ui.message.saveOK.description", 2000, Severity.INFO);
    }
    
    @Listen("onClick = #cancelButton")
    public void cancelChanges() {
        processComposer.onCancel(group);
    }

    @Listen("onSubmit = #permissionGrid")
    public void savePermission(UserPermission userPermission) {
        permissionService.save(userPermission);
    }
    
    /**
     * Redirección al proceso seleccionado, obtenemos todos los hijos del grupo y de los procesos nos quedamos con el seleccionado para
     * luego mostrar la vista del proceso
     **/
    @Listen("onGoToProcessDefinition = #processGrid")
    public void goToProcessForm(ProcessDefinition processDefinition) {
        
        ((ProcessComposer)processComposer).goToProcessForm(processDefinition);
    }
    
    @Listen("onClick = #addProcess")
    public void addProcessDefinition() {
        val processDefinition = new ProcessDefinition();

        ProcessPopupComposer.openWindow(processDefinition, group, new ZKCrossComposer<ProcessDefinition>() {

            @Override
            public void refresh(ProcessDefinition oldObject, ProcessDefinition newObject) {
                // Nada a realizar
            }

            @Override
            public void onSubmit(ProcessDefinition process, DataBinder dataBinder) {
                for (val param : process.getParameters()) {
                    if (param == null) {
                        val window = getPage().getFellow("processEditWindow");
                        if (window != null) {
                            throwWrongValuesI18n(window.getFellow("parametersListbox"), "ui.message.error.parameterEmpty");
                        } else {
                            throwWrongValuesI18n(parametersListbox, "ui.message.error.parameterEmpty");
                        }
                    } else {
                        validate(param, dataBinder);
                    }
                }
                validate(process, dataBinder);
                
                ((ProcessComposer)processComposer).addElementToTree(treeNode, process);
                processDefinitionModel.add(process);
            }

            @Override
            public void onCancel(ProcessDefinition process) {
                
            }
        });
    }
}
