package com.grupogodo.corp.projects.optimus.composers;

import java.util.*;

import javax.annotation.PostConstruct;

import lombok.*;

import org.hibernate.validator.constraints.NotBlank;
import org.springframework.beans.factory.annotation.Autowired;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.Page;
import org.zkoss.zk.ui.event.*;
import org.zkoss.zk.ui.event.EventListener;
import org.zkoss.zk.ui.metainfo.ComponentInfo;
import org.zkoss.zk.ui.select.annotation.Listen;
import org.zkoss.zk.ui.select.annotation.Wire;
import org.zkoss.zkplus.databind.DataBinder;
import org.zkoss.zul.*;
import org.zkoss.zul.event.TreeDataEvent;

import com.grupogodo.corp.libs.jpa.core.BaseEntity;
import com.grupogodo.corp.libs.springzk.scope.DesktopScoped;
import com.grupogodo.corp.libs.springzk.spring.*;
import com.grupogodo.corp.libs.zkcomponents.components.growl.Growl;
import com.grupogodo.corp.libs.zkcomponents.components.growl.Growl.Severity;
import com.grupogodo.corp.libs.zkcomponents.components.input.GComboBox;
import com.grupogodo.corp.libs.zkcomponents.components.listbox.GListBox;
import com.grupogodo.corp.projects.optimus.composers.popups.GroupPopupComposer;
import com.grupogodo.corp.projects.optimus.composers.popups.ProcessPopupComposer;
import com.grupogodo.corp.projects.optimus.entities.*;
import com.grupogodo.corp.projects.optimus.entities.Group;
import com.grupogodo.corp.projects.optimus.repositories.ProcessRepository;
import com.grupogodo.corp.projects.optimus.services.*;
import com.grupogodo.corp.projects.optimus.services.processor.*;
import com.grupogodo.corp.projects.optimus.services.processor.ParameterInfo.ParameterType;

@DesktopScoped
public class ProcessComposer extends ZKModelAwareComposer<Component> implements ZKCrossComposer<BaseEntity> {

    private static final long serialVersionUID = 2345427318959691259L;

    private class ProcessTreeModel extends DefaultTreeModel<BaseEntity> {

        private static final long serialVersionUID = -6919770287542593521L;

        public ProcessTreeModel(TreeNode<BaseEntity> root) {
            super(root);
        }

        public void invalidate() {
            fireEvent(TreeDataEvent.STRUCTURE_CHANGED, getPath(ghostTreeNode), 0, 0);
        }
    }

    private class ProcessItemRenderer implements TreeitemRenderer<TreeNode<BaseEntity>> {

        @Override
        public void render(Treeitem item, TreeNode<BaseEntity> node, int index) throws Exception {
            Treerow treeRow = new Treerow();
            Treecell cell = new Treecell();
            Label label = new Label();
            cell.appendChild(label);
            treeRow.appendChild(cell);

            if (node.getData() instanceof Group) {
                Group group = (Group) node.getData();
                label.setValue(group.getName());
                item.setContext("groupContext");
                cell.setImage("/resources/images/menu_tree/group.png");

            } else if (node.getData() instanceof ProcessDefinition) {
                ProcessDefinition processDefinition = (ProcessDefinition) node.getData();
                label.setValue(processDefinition.getDescription());
                item.setContext("processContext");
                cell.setImage("/resources/images/menu_tree/process.png");
            }

            Boolean openStatus = treeOpenStatusMap.get(node.getData().getId());
            item.setOpen(openStatus == null ? false : openStatus);

            item.setAttribute("id", node.getData().getId());
            label.setStyle("margin: 2px");
            item.setValue(node);
            if (item.getChildren().size() <= 0) {
                item.appendChild(treeRow);
            }

            TreeEventListener treeEventListener = new TreeEventListener();
            item.addEventListener(Events.ON_CLICK, treeEventListener);
            item.addEventListener(Events.ON_DOUBLE_CLICK, treeEventListener);
            item.addEventListener(Events.ON_RIGHT_CLICK, treeEventListener);
        }
    }

    private class TreeEventListener implements EventListener<Event> {

        @Override
        public void onEvent(Event event) throws Exception {
            Treeitem currentItem = (Treeitem) event.getTarget();
            currentNode = currentItem.getValue();

            // relacionar vistas con Formularios a mostrar, de grupo o de procesos
            if (currentNode.getData() instanceof Group) {

                newProcessButton.setDisabled(false);
                newGroupButton.setDisabled(false);

                loadGroupView();

            // Vista Process
            } else if (currentNode.getData() instanceof ProcessDefinition) {
                if (currentGroup == null) {
                    currentGroup = (Group) currentNode.getParent().getData();
                }

                newProcessButton.setDisabled(true);
                newGroupButton.setDisabled(true);

                loadProcessView();
            }
        }
    }

    // Clase de para obtener los dos tipos de parametros según el processor seleccionado
    @Getter
    public static class ParameterWrapper {

        private ParameterInfo parameterInfo;

        @Setter
        private Parameter parameter;

        public ParameterWrapper(ParameterInfo parameterInfo) {
            this.parameterInfo = parameterInfo;
        }

        public static List<ParameterWrapper> toWrapperList(List<ParameterInfo> parameters) {
            if (parameters == null) {
                return null;
            }
            List<ParameterWrapper> parameterWrapperList = new ArrayList<ParameterWrapper>(parameters.size());
            for (val parameter : parameters) {
                parameterWrapperList.add(new ParameterWrapper(parameter));
            }

            return parameterWrapperList;
        }

        public ParameterWrapper(Parameter parameter) {
            this.parameter = parameter;
        }

        public String getKey() {
            if (parameter != null) {
                return parameter.getKey();
            }
            return parameterInfo.getName();
        }
        
        public void setKey(String value) {
            if (parameter != null) {
                parameter.setKey(value);
            } else {
                parameter = new Parameter(parameterInfo.getName(), value, false);
            }
        }

        @NotBlank
        public String getValue() {
            if (parameter != null) {
                return parameter.getValue();
            }
            return null;
        }

        // si ya existe un parámetro, actualizamos el valor, si no, lo creamos
        public void setValue(String value) {
            if (parameter != null) {
                parameter.setValue(value);
            } else {
                parameter = new Parameter(parameterInfo.getName(), value, false);
            }
        }

        public String getType() {
            if (parameterInfo == null) {
                return ParameterType.STRING.toString();
            }
            return parameterInfo.getParameterType().toString();
        }

        public boolean isRequired() {
            return parameterInfo != null;
        }

        public boolean isRelation() {
            if (parameter != null) {
                return parameter.isRelation();
            }
            return parameterInfo.getParameterType() == ParameterType.RELATION;
        }
    }

    @Autowired
    private ProcessService processService;

    @Autowired
    private GroupService groupService;

    @Autowired
    private ProcessRepository processRepository;

    @Getter
    private ProcessTreeModel processTreeModel;

    @Getter
    private ProcessItemRenderer itemRenderer;

    @Getter
    private Processor processor;

    @Wire
    private Div detailRegion;

    @Wire
    private Toolbarbutton newGroupButton;

    @Wire
    private Toolbarbutton newProcessButton;

    @Wire
    private GListBox parametersListbox;

    private Map<Long, Boolean> treeOpenStatusMap = new HashMap<Long, Boolean>();

    private TreeNode<BaseEntity> ghostTreeNode;

    private TreeNode<BaseEntity> currentNode;

    private Group currentGroup;

    private ProcessDefinition currentProcess;

    @Wire
    private GComboBox processorComboProcess;

    // ojo! sólo se usa cuando esta vista se abre desde la vista de executions (acceso directo desde un fallo de ejecución)
    @Getter
    @Setter
    private String processDefinitionName;

    // antes de componer la página vamos a ver si existe el parámetro processdefinitionname (parámetro URL) para mostrar un nodo del árbol
    // abierto
    @Override
    public ComponentInfo doBeforeCompose(Page page, Component parent, ComponentInfo compInfo) {

        ComponentInfo componentInfo = super.doBeforeCompose(page, parent, compInfo);
        buildProcessModel();

        if (processDefinitionName != null && processDefinitionName.length() > 0) {
            // abrir el árbol y seleccionar el nodo que hemos pasado como processDefinitionName
            selectTreeNode(ghostTreeNode, processDefinitionName);
        }

        return componentInfo;
    }

    @Override
    public void doAfterCompose(Component component) throws Exception {
        super.doAfterCompose(component);

        if (currentNode != null) {

            if (currentNode.getData() instanceof Group) {
                // TODO: nada aftercompose solo es para mostrar el proceso que viene de ejecución, evita petada al clickar un grupo de inicio
            } else {
                Include include = new Include("/WEB-INF/views/processDefinitionForm.zul");
                include.setDynamicProperty("data", currentNode.getData());
                detailRegion.getChildren().clear();
                include.setParent(detailRegion);
            }
        }
    }

    private void selectTreeNode(TreeNode<BaseEntity> treeNode, String processDefinitionName) {

        if (treeNode == null || treeNode.getChildren() == null) {
            return;
        }

        for (TreeNode<BaseEntity> childNode : treeNode.getChildren()) {
            BaseEntity data = childNode.getData();
            if (data instanceof ProcessDefinition && ((ProcessDefinition) data).getName().equals(processDefinitionName)) {
                // abrirlo
                currentNode = childNode;
                processTreeModel.clearSelection();
                processTreeModel.addToSelection(childNode);
                return;
            }

            selectTreeNode(childNode, processDefinitionName);
        }
    }

    @PostConstruct
    @Reload
    public void init() {
        // Relacionar los parametros al seleccionar el proceso desde el combobox
//        processorModel = new ListModelList<String>(processorExecutor.getProcessors());
    }

    @Listen("onClick = #groupContextRemove")
    public void removeGroupFromTree() {

        groupService.delete((Group) currentNode.getData());
        removeCurrentElementFromTree();
    }

    @Listen("onClick = #processContextRemove")
    public void removeProcessFromTree() {

        processService.delete((ProcessDefinition) currentNode.getData());
        removeCurrentElementFromTree();
    }

    @Listen("onClick = #deleteProcessButton")
    public void removeFromTree() {

        // Window window = new Window("Confirmación", "normal", true);
        // window.appendChild(new Label("Confirmar!"));
        // window.appendChild(new Button("SI"));
        // window.appendChild(new Button("NO!"));
        // window.doModal();

        if (currentNode.getData() instanceof ProcessDefinition) {
            removeProcessFromTree();
        } else {
            removeGroupFromTree();
        }
    }

    @Listen("onClick = #groupContextAdd;onClick = #newGroupButton")
    public void addGroup() {
        val group = new Group();
        group.setParent(currentGroup);

        GroupPopupComposer.openWindow(group, new ZKCrossComposer<Group>() {

            @Override
            public void refresh(Group oldObject, Group newObject) {
                // Nada a realizar

            }

            @Override
            public void onSubmit(Group group, DataBinder dataBinder) {
                onSubmitGroup(group, dataBinder);
            }

            @Override
            public void onCancel(Group group) {
                onCancelGroup(group);
            }
        });
    }

    @Listen("onClick = #addProcess;onClick = #groupContextAddProcess;onClick = #newProcessButton")
    public void addProcessDef() {
        val processDefinition = new ProcessDefinition();

        ProcessPopupComposer.openWindow(processDefinition, currentGroup, new ZKCrossComposer<ProcessDefinition>() {

            @Override
            public void refresh(ProcessDefinition oldObject, ProcessDefinition newObject) {
                // Nada a realizar

            }

            @Override
            public void onSubmit(ProcessDefinition process, DataBinder dataBinder) {
                for (val param : process.getParameters()) {
                    if (param == null) {
                        val window = getPage().getFellow("processEditWindow");
                        if (window != null) {
                            throwWrongValuesI18n(window.getFellow("parametersListbox"), "ui.message.error.parameterEmpty");
                        } else {
                            throwWrongValuesI18n(parametersListbox, "ui.message.error.parameterEmpty");
                        }
                    } else {
                        validate(param, dataBinder);
                    }
                }
                validate(process, dataBinder);
                addElementToTree(currentNode, process);
                onSubmitProcess(process, dataBinder);
            }

            @Override
            public void onCancel(ProcessDefinition process) {
                onCancelProcess(process);
            }
        });
    }

    /**
     * Redirección al proceso seleccionado, obtenemos todos los hijos del grupo y de los procesos nos quedamos con el seleccionado para
     * luego mostrar la vista del proceso
     **/
    public void goToProcessForm(ProcessDefinition processDefinition) {
        processTreeModel.addOpenObject(currentNode);

        for (val child : currentNode.getChildren()) {
            if (child.getData().equals(processDefinition)) {
                processTreeModel.clearSelection();
                processTreeModel.addToSelection(child);
                
                detailRegion.getChildren().clear();
                Include include = ProcessDetailComposer.openForm(child, this);
                include.setParent(detailRegion);
            }
        }
    }

    // Mostrar ProcessDefinitions del grupo asociados al processor(executor) seleccionado
    public ListModelList<String> getProcessModel(String beanNameProcessor) {
        val model = new ListModelList<String>();

        if (currentNode.getData() instanceof Group) {
            // mostrar error (no deberia pasar nunca!)
            Growl.showMessageI18n("ui.message.error.noGroup", "ui.message.error.noGroup.description", 2000, Severity.ERROR);
        }

        val groupProcessDefinitions = ((Group) currentNode.getParent().getData()).getProcesses();
        for (val process : groupProcessDefinitions) {
            model.add(process.getName());
        }
        return model;
    }

    private void removeCurrentElementFromTree() {
        TreeNode<BaseEntity> parent = currentNode.getParent();
        parent.remove(currentNode);
    }

    private void buildProcessModel() {

        val rootGroup = groupService.getRoot();

        ghostTreeNode = new DefaultTreeNode<BaseEntity>(rootGroup, new LinkedList<TreeNode<BaseEntity>>());
        processTreeModel = new ProcessTreeModel(ghostTreeNode);
        itemRenderer = new ProcessItemRenderer();
        buildTreeRecursively(ghostTreeNode, rootGroup);

    }

    // Recursividad de grupos, cada grupo apunta a su parent (grupo padre) y se forma el arbol desde el primer nodo de grupo llamado
    // rootGroup.
    private void buildTreeRecursively(TreeNode<BaseEntity> parentNode, Group parent) {
        List<Group> subgroups = groupService.findByParent(parent);

        //Solo mostramos los grupos que tienen Menu = true
        for (Group group : subgroups) {
            if (group.isMenu()){                
                        
                TreeNode<BaseEntity> node = addElementToTree(parentNode, group);
                val processes = group.getProcesses();
                for (ProcessDefinition process : processes) {
                    if (!process.isDeleted()) {
                        addElementToTree(node, process);
                    }
                }
                buildTreeRecursively(node, group);
            }
        }
    }

    public TreeNode<BaseEntity> addElementToTree(TreeNode<BaseEntity> parentNode, BaseEntity entity) {
        TreeNode<BaseEntity> entityNode = null;
        if (entity instanceof ProcessDefinition) {
            entityNode = new DefaultTreeNode<BaseEntity>(entity);
        } else {
            entityNode = new DefaultTreeNode<BaseEntity>(entity, new LinkedList<TreeNode<BaseEntity>>());
        }
        parentNode.add(entityNode);
        processTreeModel.invalidate();

        return entityNode;
    }

    private void loadGroupView() {
        
        detailRegion.getChildren().clear();
        Include include = GroupDetailComposer.openForm(currentNode, this);
        include.setParent(detailRegion);
    }

    private void loadProcessView() {
        
        detailRegion.getChildren().clear();
        Include include = ProcessDetailComposer.openForm(currentNode, this);
        include.setParent(detailRegion);
    }

    public void onSubmitProcess(ProcessDefinition process, DataBinder dataBinder) {
        ProcessDefinition pd = processService.save(process);
        currentGroup.getProcesses().add(pd);
        groupService.save(currentGroup);
    }

    public void onSubmitGroup(Group group, DataBinder dataBinder) {
        validate(group, dataBinder);
        groupService.save(group);
        addElementToTree(currentNode, group);
    }

    public void onCancelProcess(ProcessDefinition process) {
        if (process.getId() != null) {
            currentProcess = processService.findOne(process.getId());
        }
    }

    public void onCancelGroup(Group group) {
        if (group.getId() != null) {
            currentGroup = groupService.findOne(group.getId());
        }
    }

    @Override
    public void onSubmit(BaseEntity baseEntity, DataBinder dataBinder) {
        if (baseEntity instanceof ProcessDefinition) {
            onSubmitProcess((ProcessDefinition) baseEntity, dataBinder);
        } else {
            onSubmitGroup((Group) baseEntity, dataBinder);
        }
    }

    @Override
    public void onCancel(BaseEntity baseEntity) {
        if (baseEntity instanceof ProcessDefinition) {
            currentNode.setData(processService.findOne(baseEntity.getId()));
            loadProcessView();
        } else {
            onCancelGroup((Group) baseEntity);
        }
    }

    @Override
    public void refresh(BaseEntity oldObject, BaseEntity newObject) {
        // TODO Auto-generated method stub
        System.out.println("BLO");
    }
}
