package com.grupogodo.corp.projects.optimus.composers;

import java.util.*;
import java.util.Map.Entry;

import javax.annotation.PostConstruct;

import lombok.*;

import org.springframework.beans.factory.annotation.Autowired;
import org.zkoss.spring.SpringUtil;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.select.annotation.Listen;
import org.zkoss.zk.ui.select.annotation.Wire;
import org.zkoss.zkplus.databind.DataBinder;
import org.zkoss.zul.*;

import com.grupogodo.corp.libs.jpa.core.BaseEntity;
import com.grupogodo.corp.libs.springzk.spring.*;
import com.grupogodo.corp.libs.zkcomponents.components.growl.Growl;
import com.grupogodo.corp.libs.zkcomponents.components.growl.Growl.Severity;
import com.grupogodo.corp.libs.zkcomponents.components.input.GComboBox;
import com.grupogodo.corp.libs.zkcomponents.components.listbox.GListBox;
import com.grupogodo.corp.projects.optimus.composers.ProcessComposer.ParameterWrapper;
import com.grupogodo.corp.projects.optimus.entities.*;
import com.grupogodo.corp.projects.optimus.entities.Group;
import com.grupogodo.corp.projects.optimus.services.ParameterService;
import com.grupogodo.corp.projects.optimus.services.ProcessService;
import com.grupogodo.corp.projects.optimus.services.processor.Processor;
import com.grupogodo.corp.projects.optimus.services.processor.ProcessorExecutor;

@org.springframework.stereotype.Component
public class ProcessDetailComposer extends ZKModelAwareComposer<Component> {

    private static final long serialVersionUID = -8052122821039164748L;

    public static Include openForm(TreeNode<BaseEntity> treeNode, ZKCrossComposer<BaseEntity> processComposer) {
        ProcessDetailComposer processDetailComposer = SpringUtil.getApplicationContext().getBean(ProcessDetailComposer.class);

        processDetailComposer.setProcessComposer(processComposer);
        processDetailComposer.setProcessDefinition((ProcessDefinition) treeNode.getData());
        processDetailComposer.setTreeNode(treeNode);

        Include include = new Include("/WEB-INF/views/processDefinitionForm.zul");
        return include;
    }
    
    @Setter(AccessLevel.PRIVATE)
    private ZKCrossComposer<BaseEntity> processComposer;
    
    @Autowired
    private ProcessService processService;

    @Autowired
    private ParameterService parameterService;

    @Autowired
    private ProcessorExecutor processorExecutor;

    @Getter
    private ListModelList<ParameterWrapper> parameterModel;
    
    @Getter
    private ListModelList<ProcessDefinition> processDefinitionModel;

    @Getter
    private ListModelList<String> processorModel;

    @Wire
    private Button addParameterButton;

    @Wire
    private GListBox parametersListbox;

    @Wire
    private GComboBox processorComboProcess;
    
    @Getter @Setter
    private ProcessDefinition processDefinition;
    
    @Setter
    private TreeNode<BaseEntity> treeNode;
    
    @PostConstruct
    @Reload
    public void init() {
        
        parameterModel = new ListModelList<ParameterWrapper>();
        processorModel = new ListModelList<String>(processorExecutor.getProcessors());
    }
    
    @Override
    public void doAfterCompose(Component component) throws Exception {
        super.doAfterCompose(component);
        
        refreshParametersListBox();
    }

    // Cuando se selecciona un processor muestra el process asociado con el bean
    @Listen("onSelect = #processorComboProcess")
    public void showParameters() {
        String processorName = processorComboProcess.getValue();
        Processor processor = processorExecutor.getProcessor(processorName);
        processDefinition.setBean(processorName);
        addParameterButton.setDisabled(!processor.isExtraParametersAllowed());

        refreshParametersListBox();
    }

//    @Listen("onDeletePermission = #permissionGrid")
//    public void onDeletePermission(UserPermission permission) {
//        permissionModel.remove(permission);
//        permissionService.delete(permission);
//    }

    @Listen("onDeleteParameter = #parametersListbox")
    public void onDeleteParameter(ParameterWrapper parameterWrapper) {
        parameterModel.remove(parameterWrapper);
        parameterService.delete(parameterWrapper.getParameter());
    }

//    @Listen("onClick = #addAuthorityButton")
//    public void addRole() {
//        val userPermission = new UserPermission("Permission name", "Description permission");
//        ((Group)data).getPermissions().add(userPermission);
//        permissionModel.clear();
//        permissionModel.addAll(((Group)data).getPermissions());
//    }

    @Listen("onClick = #addParameterButton")
    public void addParameter() {
        val parameter = new Parameter("Key name", "Valor name", true);

        processDefinition.getParameters().add(parameter);
        parameterModel.add(new ParameterWrapper(parameter));

    }
    
    // guardar los datos de processDefinitions
    @Listen("onClick = #saveButton")
    public void save(DataBinder binder) {
        processDefinition.getParameters().clear();
        for (val parameterWrapper : parameterModel) {
            if (parameterWrapper.getParameter() != null) {
                Parameter parameter = parameterWrapper.getParameter();
                parameter.setProcessDefinition(processDefinition);
                processDefinition.getParameters().add(parameter);
            }
        }
        
        processComposer.onSubmit(processDefinition, binder);
        
        Growl.showMessageI18n("ui.message.saveOk", "ui.message.saveOK.description", 2000, Severity.INFO);
    }
    
    @Listen("onClick = #cancelButton")
    public void cancelChanges() {
        processComposer.onCancel(processDefinition);
    }

//    @Listen("onSubmit = #permissionGrid")
//    public void savePermission(UserPermission userPermission) {
//        permissionService.save(userPermission);
//    }

    // Mostrar ProcessDefinitions del grupo asociados al processor(executor) seleccionado
    public ListModelList<String> getProcessModel(String beanNameProcessor) {
        val model = new ListModelList<String>();
        Set<ProcessDefinition> groupProcessDefinitions = ((Group) treeNode.getParent().getData()).getProcesses();
        for (val processDef : groupProcessDefinitions) {
            model.add(processDef.getName());
        }
        return model;
    }

    public boolean isExtraParametersAllowed() {
        return processorExecutor.getProcessor(processDefinition.getBean()).isExtraParametersAllowed();
    }
    
    private void refreshParametersListBox() {

        parameterModel.clear();

        Map<String, Parameter> informedParameters = new HashMap<String, Parameter>();
        List<ParameterWrapper> result = new LinkedList<ParameterWrapper>();

        for (Parameter processDefinitionParameter : processDefinition.getParameters()) {

            informedParameters.put(processDefinitionParameter.getKey(), processDefinitionParameter);
        }

        Processor processor = processorExecutor.getProcessor(((ProcessDefinition)processDefinition).getBean());
        
        // primero añadimos los parámetros (informados o no) obligatorios para el processor
        for (ParameterWrapper processorParameter : ParameterWrapper.toWrapperList(processor.getParameters())) {

            if (informedParameters.containsKey(processorParameter.getKey())) {
                processorParameter.setParameter(informedParameters.get(processorParameter.getKey()));
                informedParameters.remove(processorParameter.getKey());
            }

            result.add(processorParameter);
        }

        // segundo añadimos los parámetros extra si el processor los acepta y nos han venido informados
        if (processor.isExtraParametersAllowed()) {

            for (Entry<String, Parameter> parameter : informedParameters.entrySet()) {
                result.add(new ParameterWrapper(parameter.getValue()));
            }
        }

        parameterModel.addAll(result);

    }
}
