package com.grupogodo.corp.projects.optimus.composers;

import java.util.Collection;
import java.util.List;

import javax.annotation.PostConstruct;

import lombok.AllArgsConstructor;
import lombok.Getter;

import org.springframework.beans.factory.annotation.Autowired;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.select.annotation.Listen;
import org.zkoss.zkplus.databind.DataBinder;

import com.grupogodo.corp.libs.common.domain.*;
import com.grupogodo.corp.libs.springzk.scope.DesktopScoped;
import com.grupogodo.corp.libs.springzk.spring.*;
import com.grupogodo.corp.libs.zkcomponents.components.autocomplete.model.AbstractAutocompleteModel;
import com.grupogodo.corp.libs.zkcomponents.components.mesh.model.AbstractLazyModel;
import com.grupogodo.corp.projects.optimus.composers.popups.TaskPopupComposer;
import com.grupogodo.corp.projects.optimus.entities.ProcessDefinition;
import com.grupogodo.corp.projects.optimus.entities.ScheduledTask;
import com.grupogodo.corp.projects.optimus.services.ProcessService;
import com.grupogodo.corp.projects.optimus.services.TaskService;

@DesktopScoped
public class TaskComposer extends ZKModelAwareComposer<Component> implements ZKCrossComposer<ScheduledTask> {

    private static final long serialVersionUID = -3420950531693125125L;

    private class TaskModelList extends AbstractLazyModel<ScheduledTask> {

        private static final long serialVersionUID = -4967914081693002624L;

        protected TaskModelList() {

            super(10, new Order(Direction.ASC, "name"), new Order(Direction.ASC, "id"));
        }

        @Override
        protected Page<ScheduledTask> find(Collection<Filter> filters, Pageable pageable) {
            return taskService.find(filters, pageable);
        }

        @Override
        protected ScheduledTask save(ScheduledTask task) {
            return taskService.save(task);
        }

        @Override
        protected void delete(ScheduledTask task) {
            taskService.delete(task);
        }

    }

    

    @Autowired
    private TaskService taskService;

    @Autowired
    private ProcessService processService;

    @Getter
    private TaskModelList model;

    @Reload
    @PostConstruct
    public void init() {
        model = new TaskModelList();
        model.load();
    }

    @Listen("onFilter = #taskListBox")
    public void onFilter(Filter filter) {
        model.filter(filter);
    }

    @Override
    @Listen("onSubmit = #taskListBox")
    public void onSubmit(ScheduledTask task, DataBinder dataBinder) {
        validate(task, dataBinder);
        model.addOrReplace(task);

        // TODO grabar tarea
        taskService.save(task);
    }

    @Listen("onEdit = #taskListBox")
    public void onEditTask(ScheduledTask task) {
        TaskPopupComposer.openWindow(task, this);
    }

    @Listen("onDelete = #taskListBox")
    public void onDelete(ScheduledTask task) {
        model.remove(task);
    }

    @Listen("onClick = #NewTaskButton")
    public void onNewTask() {
        onEditTask(new ScheduledTask());
    }

    @Override
    @Listen("onCancel = #taskListBox")
    public void onCancel(ScheduledTask task) {
        if (task.getId() != null) {
            refresh(task, taskService.get(task.getId()));
        }
    }

    @Override
    public void refresh(ScheduledTask oldTask, ScheduledTask newTask) {
        model.replace(oldTask, newTask);
    }

}
