package com.grupogodo.corp.projects.optimus.composers.popups;

import java.util.HashMap;
import java.util.Map;

import javax.annotation.PostConstruct;

import lombok.*;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.context.annotation.ScopedProxyMode;
import org.springframework.stereotype.Component;
import org.zkoss.spring.SpringUtil;
import org.zkoss.zk.ui.select.annotation.Listen;
import org.zkoss.zkplus.databind.DataBinder;
import org.zkoss.zul.ListModelList;
import org.zkoss.zul.Window;

import com.grupogodo.corp.libs.springzk.spring.ZKCrossComposer;
import com.grupogodo.corp.libs.springzk.spring.ZKModelAwareComposer;
import com.grupogodo.corp.libs.zkcomponents.components.window.GWindow;
import com.grupogodo.corp.projects.optimus.entities.*;
import com.grupogodo.corp.projects.optimus.services.*;

@Component
@Scope(value = "prototype", proxyMode = ScopedProxyMode.NO)
public class ExecutionPopupComposer extends ZKModelAwareComposer<Window> {
    
    private static final long serialVersionUID = -8910862036480279542L;

    public static void openWindow(Execution execution, ZKCrossComposer<Execution> executionComposer) {
        ExecutionPopupComposer executionPopupComposer = SpringUtil.getApplicationContext().getBean(ExecutionPopupComposer.class);
        executionPopupComposer.setExecution(execution);
        executionPopupComposer.setExecutionComposer(executionComposer);

        Map<String, Object> parameters = new HashMap<String, Object>();
        parameters.put("composer", executionPopupComposer);
        GWindow.doModal("/WEB-INF/views/popups/executionEditPopup.zul", parameters);
    }

    
    @Setter(AccessLevel.PRIVATE)
    private ZKCrossComposer<Execution> executionComposer;
    
       
    @Autowired
    private ExecutionService executionService;
    
    @Autowired
    private ProcessService processService;
    
    @Autowired
    private TaskService taskService;
    
    @Getter
    private ListModelList<Execution> executionModel;
    
    @Getter
    private ListModelList<ProcessDefinition> processModel = new ListModelList<ProcessDefinition>();
    
    @Getter
    private ListModelList<ScheduledTask> taskModel = new ListModelList<ScheduledTask>();
    
       
    @Getter
    @Setter(AccessLevel.PRIVATE)
    private Execution execution;
    
    @Getter
    @Setter(AccessLevel.PRIVATE)
    private ProcessDefinition process;
    
    @Getter
    @Setter(AccessLevel.PRIVATE)
    private ScheduledTask scheduledTask;
    
    @Getter
    private Exception exception;
    
    @PostConstruct
    public void init() {
        processModel.addAll(processService.findAll());
        taskModel.addAll(taskService.findAll());
    }

    @Listen("onSubmit = #executionEditWindow")
    public void onSubmit(Execution execution, DataBinder dataBinder) {
        
        validate(execution, dataBinder);
        executionComposer.onSubmit(execution, dataBinder);
        //TODO grabar execution                        
    }
    
    @Listen("onCancel = #executionEditWindow")
    public void onCancel(Execution execution) {
        executionComposer.onCancel(execution);
    }       
   

}
