package com.grupogodo.corp.projects.optimus.composers.popups;

import java.util.HashMap;
import java.util.Map;

import javax.annotation.PostConstruct;

import lombok.*;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.context.annotation.ScopedProxyMode;
import org.springframework.stereotype.Component;
import org.zkoss.spring.SpringUtil;
import org.zkoss.zk.ui.select.annotation.Listen;
import org.zkoss.zkplus.databind.DataBinder;
import org.zkoss.zul.ListModelList;
import org.zkoss.zul.Window;

import com.grupogodo.corp.libs.springzk.spring.ZKCrossComposer;
import com.grupogodo.corp.libs.springzk.spring.ZKModelAwareComposer;
import com.grupogodo.corp.libs.zkcomponents.components.window.GWindow;
import com.grupogodo.corp.projects.optimus.entities.*;
import com.grupogodo.corp.projects.optimus.services.GroupService;

@Component
@Scope(value = "prototype", proxyMode = ScopedProxyMode.NO)
public class GroupPopupComposer extends ZKModelAwareComposer<Window> {
    
    private static final long serialVersionUID = -8910862036480279542L;

    public static void openWindow(Group group, ZKCrossComposer<Group> groupComposer) {
        GroupPopupComposer groupPopupComposer = SpringUtil.getApplicationContext().getBean(GroupPopupComposer.class);
        groupPopupComposer.setGroup(group);
        groupPopupComposer.setGroupComposer(groupComposer);

        Map<String, Object> parameters = new HashMap<String, Object>();
        parameters.put("composer", groupPopupComposer);
        GWindow.doModal("/WEB-INF/views/popups/groupEditPopup.zul", parameters);
    }

    
    @Setter(AccessLevel.PRIVATE)
    private ZKCrossComposer<Group> groupComposer;
    
       
    @Autowired
    private GroupService groupService;
    
    @Getter
    private ListModelList<ProcessDefinition> processModel;
    
    @Getter
    private ListModelList<UserPermission> permissionModel;
    
    @Getter
    @Setter(AccessLevel.PRIVATE)
    private Group group;
    
    @Getter
    @Setter(AccessLevel.PRIVATE)
    private ProcessDefinition process;
    
    @Getter
    private ListModelList<Parameter> parameterModel;
    
    @PostConstruct
    public void init() {
        processModel = new ListModelList<ProcessDefinition>();
        permissionModel = new ListModelList<UserPermission>(); 
        parameterModel = new ListModelList<Parameter>();
    }

    @Listen("onSubmit = #groupEditWindow")
    public void onSubmit(Group group, DataBinder dataBinder) {
        groupComposer.onSubmit(group, dataBinder);
    }
    
    @Listen("onCancel = #groupEditWindow")
    public void onCancel(Group group) {
        groupComposer.onCancel(group);
    }
    
    @Listen("onDelete = #permissionGrid; onDeletePermission = #permissionGrid")
    public void onDelete(UserPermission permission) {
        permissionModel.remove(permission);        
    }
    
    @Listen("onDeleteProcess = #processGrid")
    public void onDelete(ProcessDefinition process) {
      processModel.remove(process);        
    }
    
    @Listen("onClick = #addpermissionButton")
    public void addRole() {
        val userPermission = new UserPermission("Permission name", "Description permission");
        group.getPermissions().add(userPermission);
        permissionModel.clear();
        permissionModel.addAll(group.getPermissions());
    }
    
    @Listen("onClick = #addProcessButton")
    public void addProcess() {
        val processDefinition = new ProcessDefinition("Process name", "Executor name", "Description process", true);
        group.getProcesses().add(processDefinition);        
        processModel.clear();
        processModel.addAll(group.getProcesses());
    }

}
