package com.grupogodo.corp.projects.optimus.composers.popups;

import java.util.*;

import javax.annotation.PostConstruct;

import lombok.*;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.context.annotation.ScopedProxyMode;
import org.springframework.stereotype.Component;
import org.zkoss.spring.SpringUtil;
import org.zkoss.zk.ui.select.annotation.Listen;
import org.zkoss.zk.ui.select.annotation.Wire;
import org.zkoss.zkplus.databind.DataBinder;
import org.zkoss.zul.*;

import com.grupogodo.corp.libs.springzk.spring.ZKCrossComposer;
import com.grupogodo.corp.libs.springzk.spring.ZKModelAwareComposer;
import com.grupogodo.corp.libs.zkcomponents.components.input.GComboBox;
import com.grupogodo.corp.libs.zkcomponents.components.window.GWindow;
import com.grupogodo.corp.projects.optimus.composers.ProcessComposer.ParameterWrapper;
import com.grupogodo.corp.projects.optimus.entities.Parameter;
import com.grupogodo.corp.projects.optimus.entities.ProcessDefinition;
import com.grupogodo.corp.projects.optimus.services.ParameterService;
import com.grupogodo.corp.projects.optimus.services.processor.Processor;
import com.grupogodo.corp.projects.optimus.services.processor.ProcessorExecutor;

@Component
@Scope(value = "prototype", proxyMode = ScopedProxyMode.NO)
public class ProcessPopupComposer extends ZKModelAwareComposer<Window> {

    private static final long serialVersionUID = -8910862036480279542L;

    public static void openWindow(ProcessDefinition process, com.grupogodo.corp.projects.optimus.entities.Group group,
            ZKCrossComposer<ProcessDefinition> processComposer) {
        ProcessPopupComposer processPopupComposer = SpringUtil.getApplicationContext().getBean(ProcessPopupComposer.class);
        processPopupComposer.setProcessDefinition(process);
        processPopupComposer.setProcessComposer(processComposer);
        processPopupComposer.setGroup(group);

        Map<String, Object> parameters = new HashMap<String, Object>();
        parameters.put("composer", processPopupComposer);
        GWindow.doModal("/WEB-INF/views/popups/processEditPopup.zul", parameters);
    }

    @Autowired
    private ParameterService parameterService;

    @Autowired
    private ProcessorExecutor processorExecutor;

    @Setter(AccessLevel.PRIVATE)
    private ZKCrossComposer<ProcessDefinition> processComposer;

    @Getter
    @Setter(AccessLevel.PRIVATE)
    private ProcessDefinition processDefinition;

    @Setter
    private com.grupogodo.corp.projects.optimus.entities.Group group;

    @Getter
    private ListModelList<ParameterWrapper> parameterModel;

    @Getter
    private ListModelList<String> processorModel;

    @Getter
    private Processor processor;

    @Wire
    private Button addparameterButton;

    @Wire
    private GComboBox processorCombo;

    @PostConstruct
    public void init() {
        processDefinition = new ProcessDefinition();

        parameterModel = new ListModelList<ParameterWrapper>();
        processorModel = new ListModelList<String>(processorExecutor.getProcessors());

    }

    public ListModelList<String> getProcessModel(String beanNameProcessor) {
        val model = new ListModelList<String>();
        val groupProcessDefinitions = group.getProcesses();
        for (val process : groupProcessDefinitions) {
            model.add(process.getName());
        }
        return model;
    }

    @Listen("onDeleteParameter = #parametersListbox")
    public void onDelete(Parameter parameter) {
        parameterModel.remove(parameter);
        parameterService.delete(parameter);
    }

    @Listen("onClick = #addparameterButton")
    public void addParameter() {

        val parameter = new Parameter("Key name", "Valor name", true);

        processDefinition.getParameters().add(parameter);
        parameterModel.add(new ParameterWrapper(parameter));

    }

    @Listen("onSelect = #processorCombo")
    public void showParameters() {
        // al seleccionar el processor asociarlo al processDefinition
        String processorName = processorCombo.getValue();
        processor = processorExecutor.getProcessor(processorName);
        processDefinition.setBean(processorName);
        addparameterButton.setDisabled(!processor.isExtraParametersAllowed());

        refreshParametersListBox();
    }

    private void refreshParametersListBox() {

        parameterModel.clear();

        List<ParameterWrapper> result = new LinkedList<ParameterWrapper>();

        for (ParameterWrapper processorParameter : ParameterWrapper.toWrapperList(processor.getParameters())) {
            result.add(processorParameter);
        }

        parameterModel.addAll(result);

    }

    @Listen("onSubmit = #processEditWindow")
    public void onSubmit(ProcessDefinition process, DataBinder dataBinder) {
        process.getParameters().clear();
        for (val parameterWrapper : parameterModel) {
            process.getParameters().add(parameterWrapper.getParameter());
        }
        processComposer.onSubmit(process, dataBinder);
    }

    @Listen("onCancel = #processEditWindow")
    public void onCancel(ProcessDefinition process) {
        processComposer.onCancel(process);
    }

}
