package com.grupogodo.corp.projects.optimus.config;

import java.util.Arrays;
import java.util.List;
import java.util.Properties;

import javax.annotation.Resource;

import org.springframework.beans.factory.InitializingBean;
import org.springframework.context.ApplicationContextAware;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Import;
import org.springframework.orm.jpa.vendor.Database;
import org.springframework.transaction.annotation.EnableTransactionManagement;

import com.grupogodo.corp.libs.jpa.JpaConfig;
import com.grupogodo.corp.libs.jpa.eclipselink.DdlGeneration;
import com.grupogodo.corp.projects.optimus.entities.ProcessDefinition;
import com.grupogodo.corp.projects.optimus.repositories.impl.ProcessRepositoryImpl;

@Configuration
@Import(Config.class)
@EnableTransactionManagement
public class DaoConfig extends JpaConfig implements InitializingBean, ApplicationContextAware {

    @Resource
    private Properties properties;

    public void configure() {
        this.properties = getApplicationContext().getBean("properties", Properties.class);
        setConnection(Database.valueOf(p("datasource.db")), p("datasource.url"), p("datasource.username"), p("datasource.password"));
        setDdlGeneration(DdlGeneration.valueOf(p("eclipselink.ddl-generation")));
        setSessionCustomizer(null);
    }

    private String p(String key) {
        return properties.getProperty(key);
    }

    @Override
    public List<Package> getEntityPackages() {
        return Arrays.asList(ProcessDefinition.class.getPackage());
    }

    @Override
    public List<Package> getRepositoryPackages() {
        return Arrays.asList(ProcessRepositoryImpl.class.getPackage());
    }
}
