package com.grupogodo.corp.projects.optimus.config;

import java.util.Properties;

import javax.annotation.Resource;

import lombok.val;
import lombok.extern.slf4j.Slf4j;

import org.springframework.context.annotation.*;
import org.springframework.context.support.ReloadableResourceBundleMessageSource;
import org.springframework.web.servlet.config.annotation.*;
import org.springframework.web.servlet.i18n.LocaleChangeInterceptor;

import com.grupogodo.corp.libs.godointegration.metainfo.MetaInfoConfig;
import com.grupogodo.corp.libs.godoweb.config.*;
import com.grupogodo.corp.libs.spring.web.interceptor.ElapsedTimeInterceptor;
import com.grupogodo.corp.libs.springzk.spring.ZKToolboxConfigurer;

@Slf4j
@Configuration
@Import({ SecurityConfig.class, ZKConfig.class, MetaInfoConfig.class, MessageSourceConfig.class })
//@ImportResource("classpath:applicationContext-security.xml") //añadido
public class MvcConfig extends WebMvcConfigurerAdapter {

    private static final String ENCODING = "UTF-8";
    
    @Resource
    private Properties properties;
    
    @Bean
    public ReloadableResourceBundleMessageSource optimusMessageSource() {
        val bean = new ReloadableResourceBundleMessageSource();
        bean.setBasename("classpath:optimus");
        bean.setDefaultEncoding(ENCODING);
        bean.setAlwaysUseMessageFormat(true);
        return bean;
    }

    @Bean
    public ZKToolboxConfigurer zkToolboxConfigurer() {
        return new ZKToolboxConfigurer("classpath:zkToolbox.xml");
    }

    @Override
    public void addInterceptors(InterceptorRegistry registry) {
        registry.addInterceptor(new ElapsedTimeInterceptor());
        registry.addInterceptor(new LocaleChangeInterceptor());
    }

    @Override
    public void addResourceHandlers(ResourceHandlerRegistry registry) {
        int cache = 0;
        try {
            cache = Integer.parseInt(properties.getProperty("resources.cache"));
        } catch (Exception e) {
            log.warn("Property resources.cache not found. Setting cache to 0 seconds.");
        }
        registry.addResourceHandler("/resources/**").addResourceLocations("/resources/").setCachePeriod(cache);
        registry.addResourceHandler("/favicon.ico").addResourceLocations("/favicon.ico").setCachePeriod(cache);
    }
}
