package com.grupogodo.corp.projects.optimus.services.impl;

import java.util.*;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.grupogodo.corp.libs.common.domain.*;
import com.grupogodo.corp.projects.optimus.entities.*;
import com.grupogodo.corp.projects.optimus.repositories.ExecutionRepository;
import com.grupogodo.corp.projects.optimus.repositories.ProcessRepository;
import com.grupogodo.corp.projects.optimus.services.ExecutionService;

@Service
@Transactional(readOnly = true)
public class ExecutionServiceImpl implements ExecutionService {

    @Autowired
    private ExecutionRepository executionRepository;

    @Autowired
    private ProcessRepository processRepository;

    @Override
    public List<Execution> findAll() {

        return executionRepository.findAll();
    }

    @Override
    @Transactional
    public Execution save(Execution execution) {

        return executionRepository.save(execution);
    }

    @Override
    public void delete(Execution execution) {

        executionRepository.delete(execution);
    }

    @Override
    public List<Execution> findByGroup(Group group) {

        List<Execution> result = new LinkedList<Execution>();

        for (ProcessDefinition processDefinition : group.getProcesses()) {
            result.addAll(executionRepository.findAllByProcessDefinition(processDefinition));
        }

        return result;
    }

    @Override
    public List<Execution> findByProcessDefinition(ProcessDefinition processDefinition) {
        
        List<Execution> result = new LinkedList<Execution>();

        result.addAll(executionRepository.findAllByProcessDefinition(processDefinition));

        return result;
    }

    @Override
    public Execution get(Long id) {

        return executionRepository.findOne(id);
    }

    @Override
    public Page<Execution> find(Collection<Filter> filters, Pageable pageable) {

        return executionRepository.find(filters, pageable);
    }

}
