package com.grupogodo.corp.projects.optimus.services.impl;

import java.util.List;

import lombok.val;

import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.grupogodo.corp.projects.optimus.entities.Group;
import com.grupogodo.corp.projects.optimus.repositories.GroupRepository;
import com.grupogodo.corp.projects.optimus.services.GroupService;

@Service
@Transactional(readOnly = true)
public class GroupServiceImpl implements GroupService {

    @Autowired
    private GroupRepository groupRepository;

    @Override
    @Transactional
    public Group save(Group group) {
        return groupRepository.save(group);
    }

    @Override
    @Transactional
    public void delete(Group group) {
        group.setDeleted(true);
        groupRepository.save(group);
    }

    @Override
    public Group getRoot() {
        val groups = findByParent(null);

        if (CollectionUtils.isEmpty(groups)) {
            throw new IllegalStateException("There isn't root group in structure");
        }

        return groups.get(0);
    }

    @Override
    public List<Group> findByParent(Group parent) {
        return groupRepository.findByParent(parent);
    }

    @Override
    public List<Group> findAll() {
        return groupRepository.findAll();
    }

    @Override
    public Group findOne(Long id) {
        return groupRepository.findOne(id);
    }
}
