package com.grupogodo.corp.projects.optimus.services.impl;

import java.sql.Timestamp;
import java.util.*;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import com.grupogodo.corp.projects.optimus.entities.*;
import com.grupogodo.corp.projects.optimus.enums.ExecutionResult;
import com.grupogodo.corp.projects.optimus.services.*;

@Service
public class PopulatorServiceImpl implements PopulatorService {

    @Autowired
    private GroupService groupService;

    @Autowired
    private ProcessService processService;

    @Autowired
    private UserPermissionService permissionService;

    @Autowired
    private TaskService taskService;

    @Autowired
    private ExecutionService executionService;

    @Autowired
    private ExecutionExceptionService executionExceptionService;

    @Override
    @Transactional
    public void populate() {

        Group root = new Group("Ghost Group", "Nodo raíz de grupos", true, null);
        Group lvGroup = new Group("La Vanguardia", "La Vanguardia", true, root);
        Group mdGroup = new Group("Mundo Deportivo", "Mundo Deportivo", true, root);

        Group dailyLvGroup = new Group("Diarias", "Diarias", true, lvGroup);
        Group weeklyLvGroup = new Group("Semanales", "Semanales", true, lvGroup);

        Group dailyLvCaGroup = new Group("CA", "CA", true, dailyLvGroup);
        Group dailyLvEsGroup = new Group("ES", "ES", true, dailyLvGroup);

        Group weeklyLvCaGroup = new Group("CA", "CA", true, weeklyLvGroup);
        Group weeklyLvEsGroup = new Group("ES", "ES", true, weeklyLvGroup);

        // Grupos
        // Bolsas Diarias Castellano
        Group stockIndexsGroup = new Group("Index", "Index bursàtils", true, dailyLvEsGroup);
        Group stockMercatContinuGroup = new Group("Mercat Continu", "Mercat Continu", true, dailyLvEsGroup);
        Group stockNouMercatGroup = new Group("NouMercat", "Nou Mercat", true, dailyLvEsGroup);
        Group stockBorsesInternacionalsGroup = new Group("Borses Internacionals", "Borses Internacionals", true, dailyLvEsGroup);
        Group stockFonsInversio1Group = new Group("FonsInversio1", "Fons Inversio 1", true, dailyLvEsGroup);
        Group stockFonsInversio2Group = new Group("FonsInversio2", "Fons Inversio 2", true, dailyLvEsGroup);

        // Bolsas Diarias Catalan
        Group stockMercatContinuCatGroup = new Group("Mercat Continu", "Mercat Continu", true, dailyLvCaGroup);
        Group stockNouMercatCatGroup = new Group("NouMercat", "Nou Mercat", true, dailyLvCaGroup);
        Group stockBorsesInternacionalsCatGroup = new Group("Borses Internacionals", "Borses Internacionals", true, dailyLvCaGroup);
        Group stockFonsInversio1CatGroup = new Group("FonsInversio1", "Fons Inversio 1", true, dailyLvCaGroup);
        Group stockFonsInversio2CatGroup = new Group("FonsInversio2", "Fons Inversio 2", true, dailyLvCaGroup);

        // Bolsas Semanales Castellano
        Group stockEuroStoxxWeekGroup = new Group("Index", "Index bursàtils", true, weeklyLvEsGroup);
        Group stockMercatContinuWeekGroup = new Group("Mercat Continu", "Mercat Continu", true, weeklyLvEsGroup);
        Group stockNouMercatWeekGroup = new Group("NouMercat", "Nou Mercat", true, weeklyLvEsGroup);
        Group stockBorsesInternacionalsWeekGroup = new Group("Borses Internacionals", "Borses Internacionals", true, weeklyLvEsGroup);
        Group stockFonsInversio1WeekGroup = new Group("FonsInversio1", "Fons Inversio 1", true, weeklyLvEsGroup);
        Group stockFonsInversio2WeekGroup = new Group("FonsInversio2", "Fons Inversio 2", true, weeklyLvEsGroup);

        // Bolsas Semanales Catalan

        Group stockEuroStoxxWeekCatGroup = new Group("Index", "Index bursàtils", true, weeklyLvCaGroup);
        Group stockMercatContinuWeekCatGroup = new Group("Mercat Continu", "Mercat Continu", true, weeklyLvCaGroup);
        Group stockNouMercatWeekCatGroup = new Group("NouMercat", "Nou Mercat", true, weeklyLvCaGroup);
        Group stockBorsesInternacionalsWeekCatGroup = new Group("Borses Internacionals", "Borses Internacionals", true, weeklyLvCaGroup);
        Group stockFonsInversio1WeekCatGroup = new Group("FonsInversio1", "Fons Inversio 1", true, weeklyLvCaGroup);
        Group stockFonsInversio2WeekCatGroup = new Group("FonsInversio2", "Fons Inversio 2", true, weeklyLvCaGroup);

        // Processos
        ProcessDefinition fundEsN1 = new ProcessDefinition("fundEsN1", "velocityProcessor", "Fons Inversió", true);
        ProcessDefinition fundCaN1 = new ProcessDefinition("fundCaN1", "velocityProcessor", "Fons Inversió", true);
        ProcessDefinition fundCaN2 = new ProcessDefinition("fundCaN2", "velocityProcessor", "Fons Inversió segona plana", true);
        // Processos comuns a les borses
        ProcessDefinition fileProcessorStock = new ProcessDefinition("fileProcessorStock", "fileProcessor", "Connexió amb fitxer PRN",
                false);
        ProcessDefinition prnProcessorStock = new ProcessDefinition("prnProcessorStock", "prnProcessor", "Obtenció de dades", false);
        ProcessDefinition velocityProcessorStock = new ProcessDefinition("velocityProcessorStock", "velocityProcessor",
                "Aplicar plantilla origen", true);

        ProcessDefinition fileProcessorStockCat = new ProcessDefinition("fileProcessorStockCat", "fileProcessor",
                "Connexió amb fitxer PRN", false);
        ProcessDefinition prnProcessorStockCat = new ProcessDefinition("prnProcessorStockCat", "prnProcessor", "Obtenció de dades", false);
        ProcessDefinition velocityProcessorStockCat = new ProcessDefinition("velocityProcessorStockCat", "velocityProcessor",
                "Aplicar plantilla cat", true);

        UserPermission admin = new UserPermission("Admin", "Permisos totales");
        UserPermission operator = new UserPermission("Operator", "Permiso ejecutor");
        UserPermission view = new UserPermission("View", "Permiso lectura");

        // creamos ejecuciones y lo asociamos a los procesos
        Date fecha = new Date(Timestamp.valueOf("2013-06-01 12:00:00").getTime());
        Execution execution1 = new Execution(fundCaN1, fecha, ExecutionResult.OK);
        execution1.setProcessDefinition(fundCaN1);

        Date fecha4 = new Date(Timestamp.valueOf("2013-06-02 13:00:00").getTime());
        Execution execution4 = new Execution(fundCaN2, fecha4, ExecutionResult.OK);
        execution4.setProcessDefinition(fundCaN2);

        Date fecha3 = new Date(Timestamp.valueOf("2013-06-03 09:00:00").getTime());
        Execution execution3 = new Execution(fundCaN1, fecha3, ExecutionResult.OK);
        execution3.setProcessDefinition(fundCaN1);

        // error en la ejecución 3
        ExecutionException executionException3 = new ExecutionException();
        executionException3.setExecution(execution3);
        execution3.setExecutionException(executionException3);
        executionException3.setProcessDefinition(execution3.getProcessDefinition());
        executionException3.setMessage("Mensaje de excepción de prueba");

        executionException3 = executionExceptionService.save(executionException3);

        Date fecha2 = new Date(Timestamp.valueOf("2013-06-01 18:30:00").getTime());
        Execution execution2 = new Execution(fundCaN1, fecha2, ExecutionResult.OK);
        execution2.setProcessDefinition(fundEsN1);

        execution1 = executionService.save(execution1);
        execution2 = executionService.save(execution2);
        execution3 = executionService.save(execution3);
        execution4 = executionService.save(execution4);

        // Añadimos los parametros de cada proceso
        fundCaN1.setParameters(Arrays.asList(new Parameter[] { new Parameter("url", "localhost", true),
                new Parameter("user", "serverggc1", true), new Parameter("template", "serverggc2", true),
                new Parameter("parametresenseRelacio", "serverggc3", false) }));
        fundEsN1.setParameters(Arrays.asList(new Parameter[] { new Parameter("url", "localhost", true),
                new Parameter("user", "serverggc4", true), new Parameter("otra cosa", "serverggc2", true),
                new Parameter("parametrosinRelacion", "serverggc3", false) }));
        // parametro template para plantilla Velocity test Fons Inversio1
        velocityProcessorStock.setParameters(Arrays.asList(new Parameter[] { new Parameter("template",
                "#set($res = ${executor.execute('prnProcessorStock')})\n" + "[NO_TRAD]##\n" + "#foreach($r in $res)\n"
                        + "#if ($r.size()==5)\n" + "#foreach($c in $r)\n" + "${c}<TB>##\n" + "#end\n" + "\n" + "#else\n"
                        + "[BOLFONTIT][BOLFONNOM]<HR0,0.3,12,0><TSL74.6,R+26,R+26,R+23.5,R+26><EM><EL-12.4>\n"
                        + "${r[0]}[/BOLFONNOM]<TH>${r[1]}<QL>[/BOLFONTIT]\n" + "#end\n" + "#end", true) }));

        // parametro file para que usará prnprocessor
        fileProcessorStock.setParameters(Arrays.asList(new Parameter[] { new Parameter("path",
                "C:\\godo\\workspace\\optimus\\misc\\exempleProces\\FD10803_origen.PRN", false) }));

        // Tareas Programadas

        ScheduledTask task1 = new ScheduledTask("prueba Bolsa Diaria", "0 0 5 31 2 ?");
        ScheduledTask task2 = new ScheduledTask("prueba Bolsa Semanal", "0 0 5 31 2 ?");

        task1.getProcessDefinitions().add(fundEsN1);
        task1.getProcessDefinitions().add(fundCaN1);
        task2.getProcessDefinitions().add(velocityProcessorStock);

        task1 = taskService.save(task1);
        task2 = taskService.save(task2);

        fundEsN1 = processService.save(fundEsN1);
        fundCaN1 = processService.save(fundCaN1);
        fundCaN2 = processService.save(fundCaN2);
        fileProcessorStock = processService.save(fileProcessorStock);
        prnProcessorStock = processService.save(prnProcessorStock);
        velocityProcessorStock = processService.save(velocityProcessorStock);
        fileProcessorStockCat = processService.save(fileProcessorStockCat);
        prnProcessorStockCat = processService.save(prnProcessorStockCat);
        velocityProcessorStockCat = processService.save(velocityProcessorStockCat);

        admin = permissionService.save(admin);
        operator = permissionService.save(operator);
        view = permissionService.save(view);

        // Añadimos procesos (cada grupo tiene asociado proceso con nombre unico) a los grupos creados

        stockFonsInversio1Group.getProcesses().add(velocityProcessorStock);
        stockFonsInversio1Group.getProcesses().add(prnProcessorStock);
        stockFonsInversio1Group.getProcesses().add(fileProcessorStock);
        stockFonsInversio1Group.getProcesses().add(fundEsN1);

        stockFonsInversio1CatGroup.getProcesses().add(velocityProcessorStockCat);
        stockFonsInversio1CatGroup.getProcesses().add(prnProcessorStockCat);
        stockFonsInversio1CatGroup.getProcesses().add(fileProcessorStockCat);
        stockFonsInversio1CatGroup.getProcesses().add(fundCaN1);

        // añadimos los permisos a los grupos
        dailyLvGroup.getPermissions().add(admin);
        weeklyLvGroup.getPermissions().add(admin);
        dailyLvEsGroup.getPermissions().add(operator);
        dailyLvCaGroup.getPermissions().add(operator);
        dailyLvEsGroup.getPermissions().add(view);
        dailyLvCaGroup.getPermissions().add(view);

        groupService.save(root);
        groupService.save(lvGroup);
        groupService.save(mdGroup);
        groupService.save(dailyLvGroup);
        groupService.save(weeklyLvGroup);
        groupService.save(dailyLvCaGroup);
        groupService.save(dailyLvEsGroup);
        groupService.save(weeklyLvCaGroup);
        groupService.save(weeklyLvEsGroup);

        groupService.save(stockIndexsGroup);
        groupService.save(stockMercatContinuGroup);
        groupService.save(stockNouMercatGroup);
        groupService.save(stockBorsesInternacionalsGroup);
        groupService.save(stockFonsInversio1Group);
        groupService.save(stockFonsInversio2Group);

        groupService.save(stockMercatContinuCatGroup);
        groupService.save(stockNouMercatCatGroup);
        groupService.save(stockBorsesInternacionalsCatGroup);
        groupService.save(stockFonsInversio1CatGroup);
        groupService.save(stockFonsInversio2CatGroup);

        groupService.save(stockEuroStoxxWeekGroup);
        groupService.save(stockMercatContinuWeekGroup);
        groupService.save(stockNouMercatWeekGroup);
        groupService.save(stockBorsesInternacionalsWeekGroup);
        groupService.save(stockFonsInversio1WeekGroup);
        groupService.save(stockFonsInversio2WeekGroup);

        groupService.save(stockEuroStoxxWeekCatGroup);
        groupService.save(stockMercatContinuWeekCatGroup);
        groupService.save(stockNouMercatWeekCatGroup);
        groupService.save(stockBorsesInternacionalsWeekCatGroup);
        groupService.save(stockFonsInversio1WeekCatGroup);
        groupService.save(stockFonsInversio2WeekCatGroup);

    }

    @Override
    public boolean isPopulated() {
        return !CollectionUtils.isEmpty(groupService.findByParent(null));
    }
}
