package com.grupogodo.corp.projects.optimus.services.processor;

import java.util.*;

import lombok.val;
import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.grupogodo.corp.projects.optimus.entities.Parameter;
import com.grupogodo.corp.projects.optimus.entities.ProcessDefinition;
import com.grupogodo.corp.projects.optimus.repositories.ProcessRepository;

/**
 * És la classe base per a operar amb els processos. És l'encarregada d'executar-los a partir del seu nom i uns paràmetres d'entrada (un Map
 * modificat)
 * 
 * @author fdeobes
 */
@Slf4j
@Component
public class ProcessorExecutor {

    @Autowired
    private Map<String, Processor> processorMap;

    @Autowired
    private ProcessRepository processRepository;

    /**
     * Returna el llistat de noms de tipus de processor. TODO: A mostrar a la combobox de Processos.executor
     * 
     * @return
     */
    public ArrayList<String> getProcessors() {
        val procs = new ArrayList<String>(processorMap.keySet());
        Collections.sort(procs);
        return procs;
    }

    public Processor getProcessor(String name) {
        return processorMap.get(name);
    }
    
    
    /**
     * Llistat de paràmetres que accepta un processor. TODO: S'ha de mostrat quan es selecciona un processor del combobox anterior.
     * 
     * @param processor
     * @return
     */
    public List<ParameterInfo> getParameters(String processor) {
        return processorMap.get(processor).getParameters();
    }

    public <T> T execute(String name, HashMap<String, Object> map, Class<T> returnType) {
        @SuppressWarnings("unchecked") T result = (T) execute(name, map, false);
        return result;
    }

    public String preview(String name, HashMap<String, Object> map) {
        return (String) execute(name, map, true);
    }

    public Object execute(String name, HashMap<String, Object> map) {
        return execute(name, map, Object.class);
    }

    public <T> T execute(String name, Class<T> returnType) {
        return execute(name, null, returnType);
    }

    public Object execute(String name) {
        return execute(name, null, Object.class);
    }

    public void exception(String message, Object... args) {
        exception(String.format(message, args));
    }

    public void exception(String message) {
        throw new ProcessorException(message);
    }

    private ProcessorContext toMap(List<Parameter> parameters) {
        val map = new ProcessorContext();
        for (val param : parameters) {
            map.put(param.getKey(), param.getValue());
        }
        return map;
    }

    private Object execute(String processDefinitionName, HashMap<String, Object> extraParams, boolean preview) {

        ProcessDefinition processDefinition = processRepository.findByName(processDefinitionName);
        
        if (processDefinition == null) {
            log.error("No se encontró la processDefinition con nombre \"" + processDefinitionName + "\".");
            return null;
        }
        
        ProcessorContext params = toMap(processDefinition.getParameters());
        if (extraParams != null) {
            params.putAll(extraParams);
        }
        
        Processor processor = processorMap.get(processDefinition.getBean());
        return preview ? processor.preview(params) : processor.execute(params);
    }

}
