package com.grupogodo.corp.projects.optimus.services.processor.impl;

import java.io.File;
import java.util.Arrays;
import java.util.List;

import lombok.SneakyThrows;

import org.springframework.stereotype.Component;

import com.grupogodo.corp.libs.util.io.parsing.*;
import com.grupogodo.corp.projects.optimus.services.processor.*;
import com.grupogodo.corp.projects.optimus.services.processor.ParameterInfo.ParameterType;

/**
 * Llegeix un fitxer de borsa i el transforma a una llista d'arrays de String.
 * 
 * @author fdeobes
 * 
 */
@Component
public class PrnProcessor extends AbstractProcessor {

    private static final String ENCODING = "Cp437";

    @Override
    public List<ParameterInfo> getParameters() {
        return Arrays.asList(new ParameterInfo("fileProcessor", ParameterType.RELATION));
    }

    @Override
    @SneakyThrows
    public List<String[]> execute(ProcessorContext context) {

        String name = context.getString("fileProcessor");
        File file = getExecutor().execute(name, File.class);

        FileTemplate fileTemplate = new FileTemplate(file, ENCODING);
        return fileTemplate.query(new RangeMatcher(-1), new ParametrizedRowMapper<String[]>() {

            @Override
            public String[] mapRow(String row, int rowNum) {
                return row.split("\\s{2,}");
            }
        });
    }

    @Override
    public String preview(ProcessorContext context) {
        StringBuilder sb = new StringBuilder("<table>");
        List<String[]> list = execute(context);
        for (String[] row : list) {
            sb.append("<tr>");
            for (String cell : row) {
                sb.append("<td>").append(cell).append("</td>");
            }
            sb.append("</tr>\n");
        }
        return sb.append("</table>").toString();

    }
}
