package com.grupogodo.corp.projects.optimus.services.processor.impl;

import java.util.*;

import javax.sql.DataSource;

import lombok.SneakyThrows;

import org.springframework.jdbc.core.simple.SimpleJdbcTemplate;
import org.springframework.stereotype.Component;

import com.grupogodo.corp.projects.optimus.services.processor.*;
import com.grupogodo.corp.projects.optimus.services.processor.ParameterInfo.ParameterType;

/**
 * Executa la query definida i retorna el resultat com una llista de mapes. Cada element de la llista correspon a una fila del resultat, i
 * cada element del mapa correspon a una cel·la. El contexte passat per paràmetre pot incloure "placeholder" per a utilitzar en la query.
 * 
 * @author fdeobes
 * 
 */
@Component
public class SqlQueryProcessor extends AbstractProcessor {

    @Override
    public List<ParameterInfo> getParameters() {
        return Arrays.asList( //
                new ParameterInfo("dataSource", ParameterType.RELATION), //
                new ParameterInfo("query", ParameterType.TEXT));
    }

    @Override
    @SneakyThrows
    public List<Map<String, Object>> execute(ProcessorContext map) {
        String name = map.getString("dataSource");
        DataSource ds = getExecutor().execute(name, DataSource.class);
        SimpleJdbcTemplate jdbc = new SimpleJdbcTemplate(ds);
        return jdbc.queryForList(map.getString("query"), map);
    }

    @Override
    public String preview(ProcessorContext context) {
        StringBuilder sb = new StringBuilder("<table>");
        List<Map<String, Object>> list = execute(context);
        if (!list.isEmpty()) {
            sb.append("<tr>");
            for (Object name : list.get(0).keySet()) {
                sb.append("<th>").append(name).append("</th>");
            }
            sb.append("</tr>");
            for (Map<String, Object> row : list) {
                sb.append("<tr>");
                for (Object cell : row.values()) {
                    sb.append("<td>").append(cell).append("</td>");
                }
                sb.append("</tr>");
            }
        }
        return sb.append("</table>").toString();
    }

}
