package com.grupogodo.corp.projects.optimus.services.processor.impl;

import java.io.StringWriter;
import java.util.Arrays;
import java.util.List;

import lombok.Delegate;
import lombok.RequiredArgsConstructor;

import org.apache.velocity.app.VelocityEngine;
import org.apache.velocity.context.Context;
import org.apache.velocity.runtime.RuntimeConstants;
import org.springframework.stereotype.Component;

import com.grupogodo.corp.libs.util.velocity.Slf4jLogChute;
import com.grupogodo.corp.projects.optimus.services.processor.*;
import com.grupogodo.corp.projects.optimus.services.processor.ParameterInfo.ParameterType;

/**
 * <p>
 * Executa un template de Velocity i en retorna l'String resultant. Aquest template pot executar altres processors utilitzant l'executor:
 * <p>
 * <code>
 *   ${executor.execute('nomDelProcess', {'par1':'val1', 'par2':'val2'})}
 * </code>
 * 
 * 
 * @author fdeobes
 * 
 */
@Component
public class VelocityProcessor extends AbstractProcessor {

    public static final String EXECUTOR_KEY = "executor";

    /**
     * Velocity necessita d'un Context, en aquesta classe adaptem el context de Velocity a ProcessContext
     * 
     * @author fdeobes
     * 
     */
    @RequiredArgsConstructor
    private class ContextMap implements Context {

        @Delegate
        private final ProcessorContext ctx;

        @Override
        public Object get(String key) {
            return ctx.get(key);
        }

        @Override
        public Object[] getKeys() {
            return ctx.keySet().toArray();
        }

    }

    private final VelocityEngine engine;

    public VelocityProcessor() {
        engine = new VelocityEngine();
        engine.setProperty(RuntimeConstants.RUNTIME_LOG_LOGSYSTEM, new Slf4jLogChute());
        engine.init();
    }

    @Override
    public boolean isExtraParametersAllowed() {
        return true;
    }

    @Override
    public List<ParameterInfo> getParameters() {
        return Arrays.asList(new ParameterInfo("template", ParameterType.TEXT));
    }

    @Override
    public String execute(ProcessorContext context) {
        String template = context.getString("template");
        StringWriter sw = new StringWriter();
        ContextMap map = new ContextMap(context);
        map.put(EXECUTOR_KEY, getExecutor());
        engine.evaluate(map, sw, "template", template);
        return sw.toString();
    }

}
