package com.grupogodo.corp.projects.optimus.service.processor;

import java.nio.ByteBuffer;
import java.nio.CharBuffer;
import java.nio.charset.Charset;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.Arrays;

import lombok.SneakyThrows;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;

import com.grupogodo.corp.projects.optimus.config.ServiceConfig;
import com.grupogodo.corp.projects.optimus.entities.Parameter;
import com.grupogodo.corp.projects.optimus.entities.ProcessDefinition;
import com.grupogodo.corp.projects.optimus.services.ProcessService;
import com.grupogodo.corp.projects.optimus.services.processor.ProcessorExecutor;

@RunWith(SpringJUnit4ClassRunner.class)
@ContextConfiguration(classes = ServiceConfig.class)
public class PrnTest {

    /**
     * Executor es la classe bàsica per operar amb els processos
     */
    @Autowired
    private ProcessorExecutor executor;

    @Autowired
    private ProcessService service;

    /**
     * Test per a guardar 3 entitats process definition i en executem una, guardem la informació a la bbdd i executem un process, simula que
     * l'usuari Transformador ha inserit la informació de la borsa econòmica desde la pantalla i l'ha executat.
     * 
     */

    @Test
    public void test1() {

        addCommonProcessors();

        /**
         * Executa una plantilla de Velocity : Demana el resultat del fiPrnProcessor e itera damunt d'ell per escriure les dades
         */
        ProcessDefinition process = new ProcessDefinition();
        process = new ProcessDefinition();
        process.setName("fiVelocityProcess");
        process.setBean("velocityProcessor");
        process.setParameters(Arrays.asList(par("template",
                "#set($res = ${executor.execute('fiPrnProcessor')}) Result: #foreach($r in $res) .\n #foreach($c in $r) $c \t #end #end")));
        service.save(process);

        /**
         * Fem un preview, executa el process i del resutat obtenim un string (velocity) i el mostrem per pantalla
         */
        String result = executor.preview("fiVelocityProcess", null);
        System.out.println(result);
    }

    @Test
    public void test2() {

        addCommonProcessors();

        String template = getStringFromFile("src/test/resources/hermesTemplate.vm", "UTF-8");

        /**
         * Executa una plantilla de Velocity : Demana el resultat del fiPrnProcessor e itera damunt d'ell per escriure les dades
         */
        ProcessDefinition process = new ProcessDefinition();
        process = new ProcessDefinition();
        process.setName("velocityHermesProcess");
        process.setBean("velocityProcessor");
        process.setParameters(Arrays.asList(par("template", template)));
        service.save(process);

        /**
         * Fem un preview, executa el process i del resutat obtenim un string (velocity) i el mostrem per pantalla
         */
        String result = executor.preview("velocityHermesProcess", null);
        System.out.println(result);
    }

    private void addCommonProcessors() {

        /**
         * Obtè un fitxer a partir de la seva ruta relativa
         */
        ProcessDefinition process = new ProcessDefinition();
        process.setName("fiFileProcessor");
        process.setBean("fileProcessor");
        process.setParameters(Arrays.asList(par("path", "misc/exempleProces/FD10803_origen.PRN")));
        service.save(process);

        /**
         * Llegir el fitxer i convertir les dades del fitxer a una estructura més manipulable (List de Array de String)
         */
        process = new ProcessDefinition();
        process.setName("fiPrnProcessor");
        process.setBean("prnProcessor");
        process.setParameters(Arrays.asList(par("fileProcessor", "fiFileProcessor")));
        service.save(process);

    }

    private Parameter par(String key, String value) {
        Parameter par = new Parameter();
        par.setKey(key);
        par.setValue(value);
        return par;
    }

    @SneakyThrows
    private String getStringFromFile(String fileName, String charset) {
        Charset cs = Charset.forName(charset);
        byte[] bytes = Files.readAllBytes(Paths.get(fileName));
        ByteBuffer bb = ByteBuffer.wrap(bytes);
        CharBuffer cb = cs.decode(bb);
        return cb.toString();
    }
}
